/**
 * Dryer
 */
#include "main.h"
#include "sensor.h"
#include "rtos/rtos.h"
#include "ssd1306xled/font6x8.h"
#include "ssd1306xled/font8x16.h"
#include "ssd1306xled/ssd1306xled.h"
#include "ssd1306xled/ssd1306xledtx.h"

/* Defines */

/* Variables */

/* Function prototypes */
static void board_Init(void);

int main(void) {
  board_Init();
  RTOS_Init();
  //AHT10_Init();
  tdelay_ms(40);
  ssd1306_init();

PORTB |= 0x20; // Led On
  ssd1306_clear();
  ssd1306_fill2(0x80, 0x00);
  tdelay_ms(500);
  ssd1306tx_stringxy((uint8_t const *)ssd1306xled_font8x16data, 0, 0, "Hello, World! :)");
  ssd1306_setpos(7, 2);
  ssd1306tx_string("SSD1306xLED Library");
  ssd1306_setpos(0, 3);
  ssd1306tx_string("!\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[5]^_`");
  ssd1306tx_stringxy((uint8_t const *)ssd1306xled_font8x16data, 0, 5, "TINUSAUR Project");
  ssd1306_setpos(7, 7);
  ssd1306tx_string("http://tinusaur.org");

//  RTOS_SetTask(AHT10_StartMeasure, 0, 500);
  aht10_t Sensor;
  aht10_st_t status;

  do {

    RTOS_DispatchTask();

    status = AHT10_StartMeasure();
    tdelay_ms(200);
    status = AHT10_GetData(&Sensor);
    if (status == AHT10_St_OK) {
      ssd1306_setpos(1, 3);
      ssd1306tx_numdec(Sensor.Temperature);
    }

    // делать нечего -- спим, ждём прерывание
#if defined(__GNUC__)
    set_sleep_mode(SLEEP_MODE_IDLE);
    sleep_mode();
#elif defined(__ICCAVR__)
    MCUCR = 1<<SE;
    __sleep();
#endif
  } while (1);
}

/**
 *  S u b r o u t i n e s
 */

/**
 * @brief Initialize periphery
 */
static void board_Init(void) {
  /* power off Analog Comparator */
  ACSR = ACD;

  /* I/O-Ports */
  DDRB = 0x22; // PB2 - Triac control, PB5 - Green LED
  DDRC = 0x30; // I2C output
  DDRD = 0x03; // D2..D7 input / UART output
  PORTB = 0x02; // close Triac
  PORTC = 0x3e; // I2C = 1, enable pull-up for switch
  PORTD = 0xfc; // enable pull-up for switch

  /* Timer0 - RTOS & tdelay_ms() */
  TCCR0A = ((0<<CS02)|(1<<CS01)|(1<<CS00));
  TCNT0 = TIMER0_CNT;
  TIMSK0 |= (1<<TOIE0);

  /* Timer1 */

  /* Timer2 - refresh screen values */
  //TCCR2 = 0x00;
  //TCNT2 = 0x00;
  //TIMSK |= _BV(TOIE2);

  /* TWI / I2C, 400 kHz mode, Prescaler=1
     ((F_CPU / F_TWI) - 16) / 2
     ((8000000 / 400000) - 16) / 2 = 2 */
  TWBR = 2;

#ifdef USE_UART
  /* USART */
  DDRD |= _BV(PD0); // RX to Input
  // Set baud rate
  UBRRH = (BAUD_PRESCALE >> 8);
  UBRRL = BAUD_PRESCALE;
  #if (BAUD_U2X != 0)
  // Asynchronous Double Speed
  UCSRA = 1 << U2X;
  #endif // BAUD_U2X
  // Turn on USART hardware (no RX, TX)
  UCSRB = (1 << RXEN) | (1 << TXEN);
  // 8 bit char sizes, 1 stop bit
  //UCSRC = (1 << URSEL) | (1 << UCSZ0) | (1 << UCSZ1); -- default value
#endif // USE_UART

  /* Enable Interrupts */
#if defined(__GNUC__)
  sei();
#elif defined(__ICCAVR__)
  __enable_interrupt();
#endif
}

/**
 * I n t e r r u p t s
 */
#if defined(__GNUC__)
/**
 * @brief заглушка для неиспользуемых прерываний
 */
ISR(__vector_default,ISR_NAKED) {
  reti();
}
#endif
